"""Add parent_run column in run table

Revision ID: 24bd81ed7dad
Revises: 1ec1eabe45bf
Create Date: 2024-05-24 10:28:38.778180

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = "24bd81ed7dad"
down_revision: Union[str, None] = "1ec1eabe45bf"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("runs", sa.Column("parent_run", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("runs", "parent_run")
    # ### end Alembic commands ###
