"""update_run_table

Revision ID: 2e57fc6dd873
Revises: 825d896d998a
Create Date: 2024-03-04 15:03:48.187472

"""

from typing import Sequence, Union

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision: str = "2e57fc6dd873"
down_revision: Union[str, None] = "825d896d998a"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "runs", sa.Column("return_code", sa.Integer(), nullable=True)
    )
    op.add_column("runs", sa.Column("message", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("runs", "message")
    op.drop_column("runs", "return_code")
    # ### end Alembic commands ###
